package org.xmldb.api.base;

/*
 *  The XML:DB Initiative Software License, Version 1.0
 *
 *
 * Copyright (c) 2000-2003 The XML:DB Initiative.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        XML:DB Initiative (http://www.xmldb.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "XML:DB Initiative" must not be used to endorse or
 *    promote products derived from this software without prior written
 *    permission. For written permission, please contact info@xmldb.org.
 *
 * 5. Products derived from this software may not be called "XML:DB",
 *    nor may "XML:DB" appear in their name, without prior written
 *    permission of the XML:DB Initiative.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the XML:DB Initiative. For more information
 * on the XML:DB Initiative, please see <http://www.xmldb.org/>.
 */

import java.io.PrintStream;
import java.io.PrintWriter;

/**
 * XMLDBException is thrown for all errors in the XML:DB API. It contains two
 * error codes one an XML:DB error code as defined in ErrorCodes and one vendor
 * specific. If the error being thrown is only vendor specific then errorCode
 * MUST be set to ErrorCodes.VENDOR_ERROR.
 *
 * @see org.xmldb.api.base.ErrorCodes
 */ 
public final class XMLDBException extends Exception 
{
   private Throwable cause = null;
   public int errorCode = ErrorCodes.UNKNOWN_ERROR;
   public int vendorErrorCode = 0;
   
   public XMLDBException () {
   } 
   
   public XMLDBException (int errorCode) {
      this(errorCode, 0, "", null);
   }

   public XMLDBException (int errorCode, String message) {
      this(errorCode, 0, message, null);
   }

   public XMLDBException (int errorCode, int vendorErrorCode) {
      this(errorCode, vendorErrorCode, "", null);
   }

   public XMLDBException (int errorCode, int vendorErrorCode, String message) {
      this(errorCode, vendorErrorCode, message, null);
   }

   public XMLDBException (int errorCode, Throwable cause) {
      this(errorCode, 0, "", cause);
   }

   public XMLDBException (int errorCode, String message, Throwable cause) {
      this(errorCode, 0, message, cause);
   }

   public XMLDBException (int errorCode, int vendorErrorCode, Throwable cause) {
      this(errorCode, vendorErrorCode, "", cause);
   }

   public XMLDBException (int errorCode, int vendorErrorCode, String message, Throwable cause) {
      super(message);
      this.errorCode = errorCode;
      this.vendorErrorCode = vendorErrorCode;
      this.cause = cause;
   }

   public Throwable getCause() {
      return this.cause;
   }

   public void printStackTrace() {
      super.printStackTrace();
      if (this.cause != null) {
         System.err.print("Caused by: ");
         this.cause.printStackTrace();
      }
   }

   public void printStackTrace(PrintStream s) {
      super.printStackTrace(s);
      if (this.cause != null) {
         s.print("Caused by: ".getBytes());
         this.cause.printStackTrace(s);
      }
   }

   public void printStackTrace(PrintWriter s) {
      super.printStackTrace(s);
      if (this.cause != null) {
         s.print("Caused by: ");
         this.cause.printStackTrace(s);
      }
   }

}
